!*****************************************************************************
!                                                                            *
!  IMAGE.F90                                                                 *
!                                                                            *
!  This program demonstrates the Fastgraph for Windows image file display    *
!  and creation functions.                                                   *
!                                                                            *
!*****************************************************************************

integer*4 function WinMain (hInstance, hPrevInstance, lpszCmdParam, nCmdShow)
!MS$ATTRIBUTES STDCALL, ALIAS : '_WinMain@16' :: WinMain
use msfwina
use fgwin

interface
integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
integer*4 window, message, wParam, lParam
end function WindowProc
end interface

integer*4 hInstance, hPrevInstance, lpszCmdParam, nCmdShow

character*12      AppName
integer*4         iret
integer*4         window
type (T_MSG)      msg
type (T_WNDCLASS) wndclass

data AppName /'FGimage'C/

if (hPrevInstance == 0) then
   wndclass%style         = CS_HREDRAW .or. CS_VREDRAW .or. CS_OWNDC
   wndclass%lpfnWndProc   = LOC(WindowProc)
   wndclass%cbClsExtra    = 0
   wndclass%cbWndExtra    = 0
   wndclass%hInstance     = hInstance
   wndclass%hIcon         = LoadIcon(NULL,IDI_APPLICATION)
   wndclass%hCursor       = LoadCursor(NULL,IDC_ARROW)
   wndclass%hbrBackground = NULL
   wndclass%lpszMenuName  = LOC(AppName)
   wndclass%lpszClassName = LOC(AppName)
   iret = RegisterClass(wndclass)
   lpszCmdParam = lpszCmdParam
end if

window = CreateWindow(      &
   AppName,                 &  ! window class name
   'Image File Demo'C,      & ! window caption
   WS_OVERLAPPEDWINDOW,     &  ! window style
   CW_USEDEFAULT,           &  ! initial x position
   CW_USEDEFAULT,           &  ! initial y position
   CW_USEDEFAULT,           &  ! initial x size
   CW_USEDEFAULT,           &  ! initial y size
   NULL,                    &  ! parent window handle
   NULL,                    &  ! window menu handle
   hInstance,               &  ! program instance handle
   NULL)                       ! creation parameters

iret = ShowWindow(window,nCmdShow)
iret = UpdateWindow(window)

do while (GetMessage(msg,NULL,0,0))
   iret = TranslateMessage(msg)
   iret = DispatchMessage(msg)
end do

WinMain = msg%wParam
return
end

!*****************************************************************************
!                                                                            *
!  WindowProc                                                                *
!                                                                            *
!  Window procedure to handle messages sent to the window.                   *
!                                                                            *
!*****************************************************************************

integer*4 function WindowProc (window, message, wParam, lParam)
!MS$ATTRIBUTES STDCALL, ALIAS:'_WindowProc@16' :: WindowProc
use msfwina
use fgwin
integer*4 window, message, wParam, lParam

include 'image.fi'

type (T_PAINTSTRUCT) ps
integer*4 hdc, hpal
integer*4 iret

save hdc, hpal

WindowProc = 0

select case (message)

   case (WM_CREATE)
      hdc = GetDC(window)
      call fg_setdc(hdc)
      hpal = fg_defpal()
      call fg_realize(hpal)

      call fg_vbinit
      hvb = fg_vballoc(4,4)
      iret = fg_vbopen(hvb)
      call fg_vbcolors

      call fg_setcolor(25)
      call fg_fillpage
      hmenu = GetMenu(window)
      return

   case (WM_COMMAND)
      select case (wParam)
         case (IDM_BMPOPEN, IDM_BMPMAKE, IDM_BMPINFO)
            call do_bmp(wParam)
            return

         case (IDM_PCXOPEN, IDM_PCXMAKE, IDM_PCXINFO)
            call do_pcx(wParam)
            return

         case (IDM_FLICOPEN, IDM_FLICPLAY, IDM_FLICFRAME, IDM_FLICRESET, IDM_FLICINFO)
            call do_flic(wParam)
            return

         case (IDM_JPEGOPEN, IDM_JPEGINFO)
            call do_jpeg(wParam)
            return

         case (IDM_QUIT)
            iret = DestroyWindow(window)
            return
      end select

   case (WM_PAINT)
      iret = BeginPaint(window,ps)
      call fg_vbscale(0,fg_getmaxx(),0,fg_getmaxy(),0,cxClient-1,0,cyClient-1)
      iret = EndPaint(window,ps)
      return

   case (WM_SETFOCUS)
      call fg_realize(hpal)
      iret = InvalidateRect(window,NULL_RECT,.TRUE.)
      return

   case (WM_SIZE)
      cxClient = LOWORD(lParam)
      cyClient = HIWORD(lParam)
      return

   case (WM_DESTROY)
      call fg_vbclose
      call fg_vbfree(hvb1)
      call fg_vbfree(hvb2)
      call fg_vbfin
      iret = DeleteObject(hpal)
      iret = ReleaseDC(window,hdc)
      call PostQuitMessage(0)
      return

end select

WindowProc = DefWindowProc(window,message,wParam,lParam)
return
end

!*****************************************************************************
!                                                                            *
!  do_bmp                                                                    *
!                                                                            *
!  Event handlers for the items on the BMP menu.                             *
!                                                                            *
!*****************************************************************************

subroutine do_bmp(selection)
use msfwina
use fgwin
integer*4 selection

include 'image.fi'

integer*4 get_open_filename, get_save_filename
integer*4 color_depth
integer*4 hcursor
integer*4 iret

select case (selection)
   case (IDM_BMPOPEN)
      if (get_open_filename('BMP files (*.BMP)'C,'*.BMP'C) == ERR) return
      if (fg_bmphead(file_name,file_header) < 0) then
         mb_text = file_title(1:lstrlen(file_title)) // ' is not a BMP file.'C
         iret = MessageBox(GetActiveWindow(),mb_text,'Error'C, &
                MB_ICONSTOP.or.MB_OK)
         return
      end if
      colors = fg_bmppal(file_name,file_palette)
      hcursor = SetCursor(LoadCursor(NULL,IDC_WAIT))
      iret = ShowCursor(.TRUE.)
      call fg_bmpsize(file_header,cxBuffer,cyBuffer)
      call switch_buffers
      iret = fg_showbmp(file_name,0)
      call fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1)
      iret = ShowCursor(.FALSE.)
      iret = SetCursor(hcursor)
      iret = lstrcpy(open_file,file_title)

      iret = EnableMenuItem(hmenu,IDM_BMPMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPINFO,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_PCXMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_PCXINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICPLAY,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICFRAME,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICRESET,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_JPEGINFO,MF_GRAYED)
      return

   case (IDM_BMPMAKE)
      if (get_save_filename('BMP'C) == OK) then
         if (colors == 0) then
            color_depth = 24
         else if (colors == 256) then
            color_depth = 8
         else if (colors == 16) then
            color_depth = 4
         else
            color_depth = 1
         end if
         hcursor = SetCursor(LoadCursor(NULL,IDC_WAIT))
         iret = ShowCursor(.TRUE.)
         iret = fg_makebmp(0,cxBuffer-1,0,cyBuffer-1,color_depth,file_name)
         iret = ShowCursor(.FALSE.)
         iret = SetCursor(hcursor)
      end if
      return

   case (IDM_BMPINFO)
      if (colors > 0) then
         write(mb_text,'(A,I5.5,A,I5.5,A,I3.3,A)') &
            open_file(1:lstrlen(open_file))//char(10), &
            cxBuffer, 'x', cyBuffer, ' pixels'//char(10), &
            colors, ' colors'//char(0)
      else
         write(mb_text,'(A,I5.5,A,I5.5,A,A)') &
            open_file(1:lstrlen(open_file))//char(10), &
            cxBuffer, 'x', cyBuffer, ' pixels'//char(10), &
            ' 24-bit RGB'//char(0)
      end if
      call fix_message(mb_text)
      iret = MessageBox(GetActiveWindow(),mb_text,'Information'C, &
             MB_ICONINFORMATION.or.MB_OK)
      return
end select

end

!*****************************************************************************
!                                                                            *
!  do_pcx                                                                    *
!                                                                            *
!  Event handlers for the items on the PCX menu.                             *
!                                                                            *
!*****************************************************************************

subroutine do_pcx(selection)
use msfwina
use fgwin
integer*4 selection

include 'image.fi'

integer*4 get_open_filename, get_save_filename
integer*4 hcursor
integer*4 iret

select case (selection)
   case (IDM_PCXOPEN)
      if (get_open_filename('PCX files (*.PCX)'C,'*.PCX') == ERR) return
      if (fg_pcxhead(file_name,file_header) < 0) then
         mb_text = file_title(1:lstrlen(file_title)) // ' is not a PCX file.'C
         iret = MessageBox(GetActiveWindow(),mb_text,'Error'C, &
                MB_ICONSTOP.or.MB_OK)
         return
      end if
      colors = fg_pcxpal(file_name,file_palette)
      hcursor = SetCursor(LoadCursor(NULL,IDC_WAIT))
      iret = ShowCursor(.TRUE.)
      call fg_pcxsize(file_header,cxBuffer,cyBuffer)
      call switch_buffers
      call fg_move(0,0)
      iret = fg_showpcx(file_name,2)
      call fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1)
      iret = ShowCursor(.FALSE.)
      iret = SetCursor(hcursor)
      iret = lstrcpy(open_file,file_title)

      iret = EnableMenuItem(hmenu,IDM_PCXMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_PCXINFO,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICPLAY,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICFRAME,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICRESET,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_JPEGINFO,MF_GRAYED)
      return

   case (IDM_PCXMAKE)
      if (get_save_filename('PCX'C) == OK) then
         hcursor = SetCursor(LoadCursor(NULL,IDC_WAIT))
         iret = ShowCursor(.TRUE.)
         iret = fg_makepcx(0,cxBuffer-1,0,cyBuffer-1,file_name)
         iret = ShowCursor(.FALSE.)
         iret = SetCursor(hcursor)
      end if
      return

   case (IDM_PCXINFO)
      if (colors > 0) then
         write(mb_text,'(A,I5.5,A,I5.5,A,I3.3,A)') &
            open_file(1:lstrlen(open_file))//char(10), &
            cxBuffer, 'x', cyBuffer, ' pixels'//char(10), &
            colors, ' colors'//char(0)
      else
         write(mb_text,'(A,I5.5,A,I5.5,A,A)') &
            open_file(1:lstrlen(open_file))//char(10), &
            cxBuffer, 'x', cyBuffer, ' pixels'//char(10), &
            ' 24-bit RGB'//char(0)
      end if
      call fix_message(mb_text)
      iret = MessageBox(GetActiveWindow(),mb_text,'Information'C, &
             MB_ICONINFORMATION.or.MB_OK)
      return
end select

end

!*****************************************************************************
!                                                                            *
!  do_flic                                                                   *
!                                                                            *
!  Event handlers for the items on the FLI/FLC menu.                         *
!                                                                            *
!*****************************************************************************

subroutine do_flic(selection)
use msfwina
use fgwin
integer*4 selection

include 'image.fi'

integer*1 context(16)
integer*2 frames
integer*4 get_open_filename
integer*4 hcursor
integer*4 iret

save context
equivalence (frames,file_header(6))

select case (selection)
   case (IDM_FLICOPEN)
      if (get_open_filename('flic files (*.FLI,*.FLC)'C,'*.FLI;*.FLC'C) == ERR) return
      if (fg_flichead(file_name,file_header) < 0) then
         mb_text = file_title(1:lstrlen(file_title)) // &
                   ' is not an FLI or FLC file.'C
         iret = MessageBox(GetActiveWindow(),mb_text,'Error'C, &
                MB_ICONSTOP.or.MB_OK)
         return
      end if
      colors = 256
      call fg_flicsize(file_header,cxBuffer,cyBuffer)
      call switch_buffers
      iret = fg_flicopen(file_name,context)
      iret = fg_flicplay(context,1,0)
      call fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1)
      iret = lstrcpy(open_file,file_title)

      iret = EnableMenuItem(hmenu,IDM_FLICPLAY,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_FLICFRAME,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_FLICRESET,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_FLICINFO,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_PCXMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_PCXINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_JPEGINFO,MF_GRAYED)
      return

   case (IDM_FLICPLAY)
      hcursor = SetCursor(LoadCursor(NULL,IDC_WAIT))
      iret = ShowCursor(.TRUE.)
      iret = fg_showflic(file_name,0,1)
      iret = ShowCursor(.FALSE.)
      iret = SetCursor(hcursor)
      iret = fg_flicskip(context,-1)
      return

   case (IDM_FLICFRAME)
      if (fg_flicplay(context,1,0) == 0) then
         iret = fg_flicskip(context,-1)
         iret = fg_flicplay(context,1,0)
      end if
      call fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1)
      return

   case (IDM_FLICRESET)
      iret = fg_flicskip(context,-1)
      iret = fg_flicplay(context,1,0)
      call fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1)
      return

   case (IDM_FLICINFO)
      write(mb_text,'(A,I5.5,A,I5.5,A,I3.3,A)') &
         open_file(1:lstrlen(open_file))//char(10), &
         cxBuffer, 'x', cyBuffer, ' pixels'//char(10), &
         frames, ' frames'//char(0)
      call fix_message(mb_text)
      iret = MessageBox(GetActiveWindow(),mb_text,'Information'C, &
             MB_ICONINFORMATION.or.MB_OK)
      return
end select

end

!*****************************************************************************
!                                                                            *
!  do_jpeg                                                                   *
!                                                                            *
!  Event handlers for the items on the JPEG menu.                            *
!                                                                            *
!*****************************************************************************

subroutine do_jpeg(selection)
use msfwina
use fgwin
integer*4 selection

include 'image.fi'

integer*4 get_open_filename
integer*4 hcursor
integer*4 iret

integer*1,allocatable::buffer(:)
integer*4 buffsize

select case (selection)
   case (IDM_JPEGOPEN)
      if (get_open_filename('JPEG files (*.JPG)'C,'*.JPG') == ERR) return
      if (fg_jpeghead(file_name,file_header) < 0) then
         mb_text = file_title(1:lstrlen(file_title)) // ' is not a JPEG file.'C
         iret = MessageBox(GetActiveWindow(),mb_text,'Error'C, &
                MB_ICONSTOP.or.MB_OK)
         return
      end if
      colors = 0
      hcursor = SetCursor(LoadCursor(NULL,IDC_WAIT))
      iret = ShowCursor(.TRUE.)
      buffsize = fg_jpegmem(file_header)
      allocate (buffer(buffsize))
      call fg_jpegbuf(buffer,buffsize)
      call fg_jpegsize(file_header,cxBuffer,cyBuffer)
      call switch_buffers
      iret = fg_showjpeg(file_name,0)
      deallocate (buffer)
      call fg_vbscale(0,cxBuffer-1,0,cyBuffer-1,0,cxClient-1,0,cyClient-1)
      iret = ShowCursor(.FALSE.)
      iret = SetCursor(hcursor)
      iret = lstrcpy(open_file,file_title)

      iret = EnableMenuItem(hmenu,IDM_JPEGINFO,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_BMPINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_PCXMAKE,MF_ENABLED)
      iret = EnableMenuItem(hmenu,IDM_PCXINFO,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICPLAY,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICFRAME,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICRESET,MF_GRAYED)
      iret = EnableMenuItem(hmenu,IDM_FLICINFO,MF_GRAYED)
      return

   case (IDM_JPEGINFO)
      write(mb_text,'(A,I5.5,A,I5.5,A,A)') &
         open_file(1:lstrlen(open_file))//char(10), &
         cxBuffer, 'x', cyBuffer, ' pixels'//char(10), &
         ' 24-bit RGB'//char(0)
      call fix_message(mb_text)
      iret = MessageBox(GetActiveWindow(),mb_text,'Information'C, &
             MB_ICONINFORMATION.or.MB_OK)
      return
end select

end

!*****************************************************************************
!                                                                            *
!  fix_message()                                                             *
!                                                                            *
!  Compress the specified null-terminated string by removing the leading     *
!  zeroes from numeric values in the string.                                 *
!                                                                            *
!*****************************************************************************

subroutine fix_message(string)
character*(*) string

integer*4 i, j
logical*1 copy_zeroes

i = 1
j = 1
copy_zeroes = .FALSE.

do while (string(i:i) /= char(0))
   if (string(i:i) < '0' .or. string(i:i) > '9') then
      string(j:j) = string(i:i)
      j = j + 1
      copy_zeroes = .FALSE.
   else if (copy_zeroes .or. string(i:i) /= '0') then
      string(j:j) = string(i:i)
      j = j + 1
      copy_zeroes = .TRUE.
   end if
   i = i + 1
end do

string(j:j) = char(0)

return
end

!*****************************************************************************
!                                                                            *
!  get_open_filename                                                         *
!                                                                            *
!  Display a dialog box that selects a list of file names that match the     *
!  specified file name. This function uses the Open File dialog box from the *
!  Windows common dialog box library.                                        *
!                                                                            *
!  The selected file name is stored with full path information in the global *
!  variable file_name, and without path information in file_title.           *
!                                                                            *
!*****************************************************************************

integer*4 function get_open_filename(matchtype, matchfile)
use msfwina
character*(*) matchtype, matchfile

include 'image.fi'

type (T_OPENFILENAME) fn
integer*4 i, j
character*128 filter

! construct file name filter string
i = lstrlen(matchtype)
j = lstrlen(matchfile)
filter = matchtype(1:i) // char(0) // matchfile(1:j) // char(0) // char(0)
default_file = char(0)

! fill in structure fields for Open File dialog box
fn%lStructSize       = 76 ! sizeof(OPENFILENAME)
fn%hwndOwner         = GetActiveWindow()
fn%lpstrFilter       = LOC(filter)
fn%lpstrCustomFilter = NULL
fn%nFilterIndex      = 1
fn%lpstrFile         = LOC(default_file)
fn%nMaxFile          = 256
fn%lpstrFileTitle    = LOC(file_title)
fn%nMaxFileTitle     = 16
fn%lpstrInitialDir   = NULL
fn%lpstrTitle        = NULL
fn%Flags             = OFN_READONLY
fn%lpstrDefExt       = LOC(matchfile(3:3))

! activate the Open File dialog box
if (GetOpenFileName(fn)) then
   i = lstrcpy(file_name,default_file)
   get_open_filename = OK
else
   get_open_filename = ERR
end if

return
end

!*****************************************************************************
!                                                                            *
!  get_save_filename                                                         *
!                                                                            *
!  Display a dialog box that selects a list of file names that match the     *
!  specified file name. This function uses the Open File dialog box from the *
!  Windows common dialog box library.                                        *
!                                                                            *
!  The selected file name is stored with full path information in the global *
!  variable file_name, and without path information in file_title.           *
!                                                                            *
!*****************************************************************************

integer*4 function get_save_filename(extension)
use msfwina
character*(*) extension

include 'image.fi'

type (T_OPENFILENAME) fn
integer*4 i

! construct default file name for the dialog box edit control

i = lstrcpy(default_file,file_title)
do i = 1,lstrlen(file_title)
   if (default_file(i:i) == '.') default_file(i:i) = char(0)
end do

! fill in structure fields for Open File dialog box
fn%lStructSize       = 76 ! sizeof(OPENFILENAME)
fn%hwndOwner         = GetActiveWindow()
fn%lpstrFilter       = NULL
fn%lpstrCustomFilter = NULL
fn%lpstrFile         = LOC(default_file)
fn%nMaxFile          = 256
fn%lpstrFileTitle    = LOC(file_title)
fn%nMaxFileTitle     = 16
fn%lpstrInitialDir   = NULL
fn%lpstrTitle        = NULL
fn%Flags             = OFN_HIDEREADONLY .or. OFN_OVERWRITEPROMPT .or. &
                       OFN_PATHMUSTEXIST
fn%lpstrDefExt       = LOC(extension)

! activate the Open File dialog box
if (GetSaveFileName(fn)) then
   i = lstrcpy(file_name,default_file)
   get_save_filename = OK
else
   get_save_filename = ERR
end if

return
end

!*****************************************************************************
!                                                                            *
!  switch_buffers                                                            *
!                                                                            *
!  Close the and release the active virtual buffer, then create and open a   *
!  new virtual buffer to hold the new image file.                            *
!                                                                            *
!*****************************************************************************

subroutine switch_buffers
use fgwin

include 'image.fi'

integer*4 iret

call fg_vbclose
call fg_vbfree(hvb)
if (colors == 0) then
   call fg_vbdepth(24)
else
   call fg_vbdepth(8)
end if
hvb = fg_vballoc(cxBuffer,cyBuffer)
iret = fg_vbopen(hvb)
call fg_vbcolors

return
end
